/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/filechooser/SPFileChooser.java,v 1.1.1.1 2001/05/15 06:04:39 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:39 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.filechooser;

import java.io.File;
import java.util.ArrayList;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;

import org.apache.commons.httpclient.HttpClient;
import org.apache.webdav.ui.filechooser.dir.*;

/**
 * Title:        SPFileChooser.java
 * Description:  A Web File Chooser.
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class SPFileChooser extends JFileChooser {

    private SPDirectoryService directoryService;

    /*
     * The first Factory Method
     */
    public static SPFileChooser createWebFileChooser(HttpClient client,
            String rootUrlPath) throws Exception {

        try {
            SPDirectoryService dirService =
                    new SPDirectoryService(client, rootUrlPath);

            SPFileSystemView fileSystemView = new SPFileSystemView(dirService);
            SPFileView fileView = new SPFileView(dirService);

            return new SPFileChooser(fileSystemView, fileView, dirService);
        } catch (Exception e) {
            throw e;
        }
    }

    /*
     * The second Factory Method
     */
    public static SPFileChooser createWebFileChooser(SPDirectoryService aDirectoryService)  {
        SPFileSystemView fileSystemView = new SPFileSystemView(aDirectoryService);
        SPFileView fileView = new SPFileView(aDirectoryService);
        return new SPFileChooser(fileSystemView, fileView, aDirectoryService);
    }

    private SPFileChooser(SPFileSystemView aFileSystemView,
                         SPFileView fileView,
                         SPDirectoryService aDirectoryService)  {
        super(aDirectoryService.getRoot(), aFileSystemView);
        setFileView(fileView);
        this.directoryService = aDirectoryService;

        setFileSelectionMode(JFileChooser.FILES_ONLY);

    }

    public SPDirectoryService getDirectoryService() {
        return directoryService;
    }

    /*
     * Given a filterDescription, this function creates a FileFilter
     * and add it as a choosable FileFilter in the file chooser.
     *
     * @param filterDescription which contains a description of the filter and
     *        a list of filtered file extensions in braces
     *        Example: filterDescription1 = "XML files (.xml)"
     *                 filterDescription2 = "Image files (.gif, .jpg)"
     */
    public void addFileFilter(String filterDescription) {
        if (isFilterSyntaxCorrect(filterDescription)) {
            FileFilter aFilter =
                    new SPFileFilter(directoryService, filterDescription);
            addChoosableFileFilter(aFilter);
        }
    }

    /**
     * Sets the current file filter. The file filter is used by the
     * file chooser to filter out files from the user's view.
     *
     * @param filterDescription which contains a description of the filter and
     *        a list of filtered file extensions in braces
     *        Example: filterDescription1 = "XML files (.xml)"
     *                 filterDescription2 = "Image files (.gif, .jpg)"
     */
    public void setFileFilter(String filterDescription) {
        if (isFilterSyntaxCorrect(filterDescription)) {
            FileFilter aFilter =
                    new SPFileFilter(directoryService, filterDescription);
            setFileFilter(aFilter);
        }
    }

    private boolean isFilterSyntaxCorrect(String filterDescription) {
        boolean   isCorrect = false;
        int openingBrace = filterDescription.indexOf("(");
        int closingBrace = filterDescription.indexOf(")");

        if ((closingBrace > openingBrace) && (openingBrace >= 0)) {
            //there must me opening and closing braces
            String extList = filterDescription.substring(openingBrace+1, closingBrace);
            extList = extList.trim();

            if ( (extList.length() >= 2) && (extList.indexOf(".") >= 0) ) {
                //at least 2 characters and there is a "."; for example: .x
                isCorrect = true;
            }
        }
        return isCorrect;
      }

}// End of SPFileChooser class









