/*
 * $Header: /home/cvspublic/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lab/HostFrame.java,v 1.1.1.1 2001/05/15 06:04:43 remm Exp $
 * $Revision: 1.1.1.1 $
 * $Date: 2001/05/15 06:04:43 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lab;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import java.io.File;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpStatus;
import org.apache.webdav.lib.*;
import org.apache.webdav.lib.methods.*;
import org.apache.webdav.ui.lib.methods.*;

import org.apache.webdav.ui.filechooser.dir.SPDirectoryService;
import org.apache.webdav.ui.filechooser.SPFileChooser;

/**
 * Title:        HostFrame.java
 * Description:  Creates a lab frame for a given webdav client
 * Company:      SpeedLegal Holdings Inc.
 * @author       Jojada J. Tirtowidjojo
 * @version 1.0
 */

public class HostFrame extends JFrame {
    HttpClient client;

    JPanel contentPane;
    BorderLayout contentPaneLayout = new BorderLayout();
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu();
    JMenuItem exitMenuItem = new JMenuItem();
    JMenu helpMenu = new JMenu();
    JMenuItem helpAboutMenuItem = new JMenuItem();
    JToolBar jToolBar1 = new JToolBar();
    JButton saveFileButton = new JButton();
    ImageIcon image1;
    ImageIcon image2;
    JButton openFileButton = new JButton();
    JPanel statusPane = new JPanel();
    JLabel statusTextLabel = new JLabel();
    JLabel statusCodeLabel = new JLabel();
    GridLayout gridLayout1 = new GridLayout();
    JSplitPane centreSplitPane = new JSplitPane();
    JPanel centreLeftPane = new JPanel();
    BorderLayout centreLeftPaneLayout = new BorderLayout();
    JLabel execStatusBar = new JLabel();
    JPanel inputPane = new JPanel();
    JLabel pathLabel = new JLabel();
    JLabel emptyLabel = new JLabel();
    JLabel methodLabel = new JLabel();
    GridBagLayout inputPaneLayout = new GridBagLayout();
    JButton cancelButton = new JButton();
    JTextField pathField = new JTextField();
    JButton okButton = new JButton();
    JComboBox methodBoxField;
    JScrollPane textScrollPane = new JScrollPane();
    JTextArea outputTextArea = new JTextArea();
    JMenuItem openMenuItem = new JMenuItem();
    JMenuItem saveMenuItem = new JMenuItem();
    JMenuItem saveAsMenuItem = new JMenuItem();
    JMenuItem lockMenuItem = new JMenuItem();
    JMenuItem unlockMenuItem = new JMenuItem();
    SPFileChooser fileChooser;
    SPDirectoryService directoryService;
    File currFile = null;  // null means new / untitled.
    boolean dirty = false;  // True means modified text.
    String currentPath;

    private boolean focusIsSet = false;
    private Frame owner;

    public HostFrame(Frame owner, HttpClient client,
                     SPFileChooser fileChooser, String currentPath) {

        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try {
	    this.owner = owner;
            this.client = client;
            this.setTitle(client.getHost()+":"+client.getPort());
	    this.fileChooser = fileChooser;
	    this.directoryService = fileChooser.getDirectoryService();
	    this.currentPath = currentPath;
            jbInit();
        } catch(Exception e) {
            e.printStackTrace();
        }
    }

    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_ACTIVATED) {
	    this.setFocus();
        }
    }

    /* File | Exit action performed */
    void exitMenuItem_actionPerformed(ActionEvent e) {
        this.setVisible(false);
    }

    /* Help | About action performed */
    void helpAboutMenuItem_actionPerformed(ActionEvent e) {
        HostFrame_AboutBox dlg = new HostFrame_AboutBox(this);
        Dimension dlgSize = dlg.getPreferredSize();
        Dimension frmSize = getSize();
        Point loc = getLocation();
        dlg.setLocation((frmSize.width - dlgSize.width) / 2 + loc.x, (frmSize.height - dlgSize.height) / 2 + loc.y);
        dlg.setModal(true);
        dlg.show();
    }

    void cancelButton_actionPerformed(ActionEvent e) {
//    cancelInputFields();
    }

    void executeChosenMethod() {
        SPWebdavMethod chosenMethod = (SPWebdavMethod) methodBoxField.getSelectedItem();

        String path = (pathField.getText()).trim();
        path = (path.length() == 0 )? "/" : path;
        pathField.setText(path);

        chosenMethod.refresh();
        chosenMethod.setPath(path);

        try {
	    chosenMethod = chosenMethod.execute();

        } catch(Exception e)  {
	    e.printStackTrace();
        }
    }

    void displayResponseMessage() {
        SPWebdavMethod chosenMethod = (SPWebdavMethod) methodBoxField.getSelectedItem();

        outputTextArea.setText(chosenMethod.getMessageContent());
        statusCodeLabel.setText("Status CODE: "+chosenMethod.getStatusCode());
        statusTextLabel.setText("Status TEXT: "+chosenMethod.getStatusText());
    }

    void okButton_actionPerformed(ActionEvent e) {
        executeChosenMethod();
        displayResponseMessage();
    }

    void openMenuItem_actionPerformed(ActionEvent e) {
        fileOpen();
    }

    /* Handle the File|Open menu item. */
    void fileOpen() {
        if (fileChooser == null)  {
	    try {
		fileChooser = SPFileChooser.createWebFileChooser(client,
		        pathField.getText());
		fileChooser.addFileFilter("XML Files (.xml)");
		fileChooser.addFileFilter("DTD Files (.dtd)");
		directoryService = fileChooser.getDirectoryService();
	    } catch (Exception e) {
		e.printStackTrace();
	    }
        }

        // Use the OPEN version of the dialog, test return for Approve/Cancel
        if (JFileChooser.APPROVE_OPTION == fileChooser.showOpenDialog(this)) {
            openFile(fileChooser.getSelectedFile());
        }

        this.repaint();
    }

    void openFile(File aFile) {
        try {
	    String fileContent = directoryService.getWebFileContentAsString(aFile);
	    outputTextArea.setText(fileContent);
	    // Cache the currently opened filename for use at save time...
	    this.currFile = aFile;
	    // ...and mark the edit session as being clean
	    this.dirty = false;
	    String mode = (directoryService.isReadOnly(aFile)) ? "READ Only" : "READ/WRITE";
	    statusTextLabel.setText("Opened "+aFile.getPath()+"  Mode: "+mode);

	    statusCodeLabel.setText(""); //clear it

        } catch (Exception e) {
	    statusTextLabel.setText("Error opening "+aFile.getPath());
	    e.printStackTrace();
        }

    }// openFile()

    /* Handle the File|Save menu item. */
    boolean fileSave() {
        int statusCode=0;
        // Handle the case where we don't have a file name yet.
        if (currFile == null) {
	    return saveAsFile();
        }

	try  {
	    String fileContent = outputTextArea.getText();
	    statusCode = directoryService.putWebFileContent(currFile, fileContent);
	    this.dirty = false;
	    if (statusCode == HttpStatus.SC_NOT_IMPLEMENTED)  {
		statusTextLabel.setText("Cancelled Saving "+currFile.getPath());
	    } else  {
		statusTextLabel.setText("Saved "+currFile.getPath());
	    }
	    statusCodeLabel.setText("Status CODE: "+statusCode);
	    return true;
	} catch (Exception e) {
	    statusTextLabel.setText("Error saving "+currFile.getPath());
	    statusCodeLabel.setText("Status CODE: "+statusCode);
	    e.printStackTrace();
	}
        return false;
    } // fileSave()

    /* Handle the File|SaveAs menu item. */
    boolean saveAsFile()  {
	if (JFileChooser.APPROVE_OPTION == fileChooser.showSaveDialog(this)) {
	    currFile = fileChooser.getSelectedFile();
	    this.repaint();
	    return fileSave();
        } else {
	    this.repaint();
	    return false;
        }
    } //saveAsFile()

    void fileLock() {
        if (currFile != null) {
	    String locktoken = directoryService.lock(currFile);
	    if (locktoken == null)  { //fails
		statusTextLabel.setText("Error locking "+currFile.getPath());
		statusCodeLabel.setText("Status CODE:");
	    } else  { //successful
		statusTextLabel.setText("Locked "+currFile.getPath());
		statusCodeLabel.setText("Status CODE:");
	    }
        }
    }

    void fileUnLock() {
        if (currFile != null) {
	    if (directoryService.unlock(currFile))  { //successful
	        statusTextLabel.setText("UnLocked "+currFile.getPath());
		statusCodeLabel.setText("Status CODE:");
	    } else  { //fails
		statusTextLabel.setText("Error unlocking "+currFile.getPath());
		statusCodeLabel.setText("Status CODE:");
	    }
        }
    }

    void saveAsMenuItem_actionPerformed(ActionEvent e)  {
        saveAsFile();
    }

    void lockMenuItem_actionPerformed(ActionEvent e)  {
        fileLock();
    }

    void unlockMenuItem_actionPerformed(ActionEvent e)  {
        fileUnLock();
    }

    void saveMenuItem_actionPerformed(ActionEvent e) {
        fileSave();
    }

    void openFileButton_actionPerformed(ActionEvent e) {
        fileOpen();
    }

    void saveFileButton_actionPerformed(ActionEvent e) {
        fileSave();
    }

  private java.net.URL getImage(String name) {
    return getClass().getResource(name);
  }


    /* Component initialization */
    private void jbInit() throws Exception  {
        initMethodBoxField();
        image1 = new ImageIcon(getImage("/images/openFile.gif"));
        image2 = new ImageIcon(getImage("/images/closeFile.gif"));

        contentPane = (JPanel) this.getContentPane();
	contentPane.setLayout(contentPaneLayout);
        this.setSize(new Dimension(800, 600));
        fileMenu.setText("File");
        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new HostFrame_exitMenuItem_ActionAdapter(this));
        helpMenu.setText("Help");
        helpAboutMenuItem.setText("About");
        helpAboutMenuItem.addActionListener(new HostFrame_helpAboutMenuItem_ActionAdapter(this));
        saveFileButton.setIcon(image2);
        saveFileButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                saveFileButton_actionPerformed(e);
            }
        });
        saveFileButton.setToolTipText("Close File");

        openFileButton.setIcon(image1);
        openFileButton.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(ActionEvent e) {
                openFileButton_actionPerformed(e);
            }
        });
        openFileButton.setToolTipText("Open File");

        statusPane.setBorder(BorderFactory.createRaisedBevelBorder());
        statusPane.setLayout(gridLayout1);
	statusPane.setPreferredSize(new Dimension(10,40));
        statusTextLabel.setFont(new java.awt.Font("SansSerif", 0, 12));
        statusTextLabel.setBorder(BorderFactory.createLoweredBevelBorder());
        statusTextLabel.setText("Status TEXT:");
        statusCodeLabel.setFont(new java.awt.Font("SansSerif", 0, 12));
        statusCodeLabel.setBorder(BorderFactory.createLoweredBevelBorder());
        statusCodeLabel.setText("Status CODE:");
        gridLayout1.setHgap(2);
        centreLeftPane.setLayout(centreLeftPaneLayout);
        execStatusBar.setText("ExecutionStatusBar");
        inputPane.setLayout(inputPaneLayout);
        pathLabel.setText("Path:");
        methodLabel.setText("Method:");
        cancelButton.setRequestFocusEnabled(false);
        cancelButton.setContentAreaFilled(false);
        cancelButton.setText("Cancel");
        cancelButton.addActionListener(new HostFrame_cancelButton_actionAdapter(this));
        okButton.setContentAreaFilled(false);
        okButton.setText("OK");
        okButton.addActionListener(new HostFrame_okButton_actionAdapter(this));
        emptyLabel.setMaximumSize(new Dimension(300, 60));
        emptyLabel.setMinimumSize(new Dimension(300, 60));
        emptyLabel.setPreferredSize(new Dimension(300, 60));
        methodBoxField.setMinimumSize(new Dimension(100, 21));
        methodBoxField.setPreferredSize(new Dimension(100, 21));
        pathField.setText(currentPath);
        centreSplitPane.setLastDividerLocation(300);
        textScrollPane.setPreferredSize(new Dimension(250, 600));
        outputTextArea.setText("jTextArea1");
        openMenuItem.setText("Open");
        openMenuItem.addActionListener(new HostFrame_openMenuItem_actionAdapter(this));
        saveMenuItem.setText("Save");
        saveMenuItem.addActionListener(new HostFrame_saveMenuItem_actionAdapter(this));
        saveAsMenuItem.setText("Save As ...");
        saveAsMenuItem.addActionListener(new HostFrame_saveAsMenuItem_actionAdapter(this));
        lockMenuItem.setText("Lock");
        lockMenuItem.addActionListener(new HostFrame_lockMenuItem_actionAdapter(this));
        unlockMenuItem.setText("UnLock");
        unlockMenuItem.addActionListener(new HostFrame_unlockMenuItem_actionAdapter(this));

        fileMenu.add(openMenuItem);
        fileMenu.add(saveMenuItem);
        fileMenu.add(saveAsMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(lockMenuItem);
        fileMenu.add(unlockMenuItem);
        fileMenu.addSeparator();
        fileMenu.add(exitMenuItem);
        helpMenu.add(helpAboutMenuItem);
        menuBar.add(fileMenu);
        menuBar.add(helpMenu);
        this.setJMenuBar(menuBar);
        contentPane.add(jToolBar1, BorderLayout.NORTH);
        jToolBar1.add(openFileButton);
        jToolBar1.add(saveFileButton, null);
        contentPane.add(statusPane, BorderLayout.SOUTH);
        statusPane.add(statusCodeLabel, null);
        statusPane.add(statusTextLabel, null);
        contentPane.add(centreSplitPane, BorderLayout.CENTER);
        centreSplitPane.add(centreLeftPane, JSplitPane.LEFT);
        centreLeftPane.add(execStatusBar, BorderLayout.SOUTH);
        centreLeftPane.add(inputPane, BorderLayout.CENTER);
        inputPane.add(methodLabel, new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 10, 0, 0), 0, 0));
        inputPane.add(pathLabel, new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 10, 0, 0), 0, 0));
        inputPane.add(emptyLabel, new GridBagConstraints(1, 3, 2,
	        GridBagConstraints.REMAINDER, 0.0, 0.0,
		GridBagConstraints.CENTER, GridBagConstraints.BOTH,
		new Insets(0, 0, 0, 0), 0, 0));
        inputPane.add(cancelButton, new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
		GridBagConstraints.SOUTH, GridBagConstraints.NONE,
		new Insets(0, 10, 0, 0), 0, 0));
        inputPane.add(methodBoxField, new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 10), 0, 0));
        inputPane.add(pathField, new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
		new Insets(0, 0, 0, 10), 0, 0));
        inputPane.add(okButton, new GridBagConstraints(2, 4, 1, 1, 0.0, 0.0,
		GridBagConstraints.SOUTHEAST, GridBagConstraints.NONE,
		new Insets(0, 0, 0, 10), 0, 0));
        centreSplitPane.add(textScrollPane, JSplitPane.RIGHT);
        textScrollPane.getViewport().add(outputTextArea, null);

	centreSplitPane.setDividerLocation(300);
    } //private jbInit()

    private void initMethodBoxField() {

	SPWebdavMethod[] availableMethods = {
            new SPCopyMethod(client),
            new SPDeleteMethod(client),
            new SPGetMethod(client),
            new SPHeadMethod(client),
            new SPLockMethod(client),
            new SPMkcolMethod(client),
            new SPMoveMethod(client),
            new SPOptionsMethod(client),
            new SPPostMethod(client),
            new SPPropFindMethod(client),
            new SPPropPatchMethod(client),
            new SPPutMethod(client),
            new SPUnlockMethod(client)
        };

        methodBoxField = new JComboBox(availableMethods);
        methodBoxField.setSelectedItem(availableMethods[9]);
    }

    private void setFocus() {
        if (!focusIsSet) {
            focusIsSet = true;
        }
    }

} //End of HostFrame class

class HostFrame_exitMenuItem_ActionAdapter implements ActionListener {
    HostFrame adaptee;

    HostFrame_exitMenuItem_ActionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.exitMenuItem_actionPerformed(e);
    }
}

class HostFrame_helpAboutMenuItem_ActionAdapter implements ActionListener {
    HostFrame adaptee;

    HostFrame_helpAboutMenuItem_ActionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.helpAboutMenuItem_actionPerformed(e);
    }
}

class HostFrame_okButton_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_okButton_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.okButton_actionPerformed(e);
    }
}

class HostFrame_cancelButton_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_cancelButton_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.cancelButton_actionPerformed(e);
    }
}

class HostFrame_openMenuItem_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_openMenuItem_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.openMenuItem_actionPerformed(e);
    }
}

class HostFrame_saveMenuItem_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_saveMenuItem_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.saveMenuItem_actionPerformed(e);
    }
}

class HostFrame_saveAsMenuItem_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_saveAsMenuItem_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.saveAsMenuItem_actionPerformed(e);
    }
}

class HostFrame_lockMenuItem_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_lockMenuItem_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.lockMenuItem_actionPerformed(e);
    }
}

class HostFrame_unlockMenuItem_actionAdapter implements java.awt.event.ActionListener {
    HostFrame adaptee;

    HostFrame_unlockMenuItem_actionAdapter(HostFrame adaptee) {
        this.adaptee = adaptee;
    }
    public void actionPerformed(ActionEvent e) {
        adaptee.unlockMenuItem_actionPerformed(e);
    }
}

