/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/content/Content.java,v 1.8 2000/12/04 01:36:26 remm Exp $
 * $Revision: 1.8 $
 * $Date: 2000/12/04 01:36:26 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.content;

import java.util.Enumeration;
import java.util.Date;
import java.util.Vector;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.lock.*;
import org.apache.slide.security.*;
import org.apache.slide.authenticate.CredentialsToken;

/**
 * Content.
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.8 $
 */
public interface Content {
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Retrieve revision descriptors.
     * 
     * @param strUri Uri
     * @return NodeRevisionDescriptors
     */
    NodeRevisionDescriptors retrieve(SlideToken token, String strUri)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        ObjectLockedException;
    
    
    /**
     * Retrieve revision descriptor.
     * 
     * @param revisionDescriptors Node revision descriptors
     * @param revisionNumber Node revision number
     */
    NodeRevisionDescriptor retrieve
        (SlideToken token, NodeRevisionDescriptors revisionDescriptors,
         NodeRevisionNumber revisionNumber)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException;
    
    
    /**
     * Retrieve revision descriptor from the latest revision 
     * in the main branch.
     * 
     * @param revisionDescriptors Node revision descriptors
     */
    NodeRevisionDescriptor retrieve
        (SlideToken token, NodeRevisionDescriptors revisionDescriptors)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException;
    
    
    /**
     * Retrieve revision descriptor.
     * 
     * @param revisionDescriptors Node revision descriptors
     * @param revisionNumber Node revision number
     */
    NodeRevisionDescriptor retrieve
        (SlideToken token, NodeRevisionDescriptors revisionDescriptors,
         String branch)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException,
        BranchNotFoundException, NodeNotVersionedException;
    
    
    /**
     * Retrieve revision content.
     * 
     * @param revisionDescriptors Node revision descriptors
     * @param revisionDescriptor Node revision descriptor
     */
    NodeRevisionContent retrieve(SlideToken token, 
                                 NodeRevisionDescriptors revisionDescriptors,
                                 NodeRevisionDescriptor revisionDescriptor)
        throws ObjectNotFoundException, AccessDeniedException, 
        RevisionNotFoundException, LinkedObjectNotFoundException, 
        ServiceAccessException, RevisionContentNotFoundException, 
        ObjectLockedException;
    
    
    /**
     * Retrieve revision content.
     * 
     * @param strUri Uri
     * @param revisionDescriptor Node revision descriptor
     */
    NodeRevisionContent retrieve(SlideToken token, String strUri, 
                                 NodeRevisionDescriptor revisionDescriptor)
        throws ObjectNotFoundException, AccessDeniedException, 
        RevisionNotFoundException, LinkedObjectNotFoundException, 
        ServiceAccessException, RevisionContentNotFoundException, 
        ObjectLockedException;
    
    
    /**
     * Create new revision descriptors.
     * 
     * @param strUri Uri
     * @param isVersioned true is the resource is versioned
     */
    public void create(SlideToken token, String strUri, 
                       boolean isVersioned)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        ObjectLockedException;
    
    
    /**
     * Create new revision in main branch.
     * 
     * @param strUri Uri
     * @param revisionDescriptor New Node revision descriptor
     * @param revisionContent New Node revision content
     */
    void create(SlideToken token, String strUri, 
                NodeRevisionDescriptor revisionDescriptor, 
                NodeRevisionContent revisionContent)
        throws ObjectNotFoundException, AccessDeniedException, 
        RevisionAlreadyExistException, LinkedObjectNotFoundException, 
        ServiceAccessException, ObjectLockedException;
    
    
    /**
     * Create new revision based on a previous revision.
     * 
     * @param strUri Uri
     * @param branch Branch in which to create the revision
     * @param newRevisionDescriptor New revision descriptor
     * @param revisionContent Node revision content
     */
    void create(SlideToken token, String strUri, String branch, 
                NodeRevisionDescriptor newRevisionDescriptor, 
                NodeRevisionContent revisionContent)
        throws ObjectNotFoundException, AccessDeniedException, 
        RevisionAlreadyExistException, LinkedObjectNotFoundException, 
        ServiceAccessException, RevisionDescriptorNotFoundException, 
        ObjectLockedException, NodeNotVersionedException, 
        BranchNotFoundException;
    
    
    /**
     * Create a branch based on specified revision.
     * 
     * @param strUri Uri
     * @param branchName Name of the new branch
     * @param basedOnRevisionDescriptor Node revision descriptor of 
     * the revision on which the new branch is based on
     */
    void fork(SlideToken token, String strUri, String branchName, 
              NodeRevisionDescriptor basedOnRevisionDescriptor)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException,
        NodeNotVersionedException, RevisionAlreadyExistException;
    
    
    /**
     * Create a branch based on specified revision.
     * 
     * @param strUri Uri
     * @param branchName Name of the new branch
     * @param basedOnRevisionNumber Node revision number of 
     * the revision on which the new branch is based on
     */
    void fork(SlideToken token, String strUri, String branchName, 
              NodeRevisionNumber basedOnRevisionNumber)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException,
        NodeNotVersionedException, RevisionAlreadyExistException;
    
    
    /**
     * Merge specified branches into a single branch.
     * 
     * @param strUri Uri
     * @param mainBranch Branch into which the other branch will be merged
     * @param branch Branch to merge into main branch
     * @param newRevisionDescriptor New revision descriptor
     * @param revisionContent Node revision content
     */
    void merge(SlideToken token, String strUri, 
               NodeRevisionDescriptor mainBranch, 
               NodeRevisionDescriptor branch, 
               NodeRevisionDescriptor newRevisionDescriptor, 
               NodeRevisionContent revisionContent)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException,
        NodeNotVersionedException, BranchNotFoundException,
        RevisionAlreadyExistException;
    
    
    /**
     * Merge specified branches into a single branch.
     * 
     * @param strUri Uri
     * @param mainBranch Branch into which the other branch will be merged
     * @param branch Branch to merge into main branch
     * @param newRevisionDescriptor New revision descriptor
     * @param revisionContent Node revision content
     */
    void merge(SlideToken token, String strUri, 
               String mainBranch, String branch, 
               NodeRevisionDescriptor newRevisionDescriptor, 
               NodeRevisionContent revisionContent)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException,
        NodeNotVersionedException, BranchNotFoundException,
        RevisionAlreadyExistException;
    
    
    /**
     * Update contents of an existing revision.
     * 
     * @param strUri Uri
     * @param revisionDescriptor Revision descriptor
     * @param revisionContent Revision content
     */
    void store(SlideToken token, String strUri, 
               NodeRevisionDescriptor revisionDescriptor,
               NodeRevisionContent revisionContent)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException, 
        RevisionNotFoundException;
    
    
    /**
     * Remove all revisions at this Uri.
     * 
     * @param revisionDescriptors Node revision descriptors
     */
    void remove(SlideToken token, 
                NodeRevisionDescriptors revisionDescriptors)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException;
    
    
    /**
     * Remove specified revision.
     * 
     * @param strUri Uri
     * @param revisionDescriptor Node revision descriptor
     */
    void remove(SlideToken token, String strUri, 
                NodeRevisionDescriptor revisionDescriptor)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException;
    
    
    /**
     * Remove specified revision.
     * 
     * @param strUri Uri
     * @param revisionNumber Revision number
     */
    void remove(SlideToken token, String strUri, 
                NodeRevisionNumber revisionNumber)
        throws ObjectNotFoundException, AccessDeniedException, 
        LinkedObjectNotFoundException, ServiceAccessException, 
        RevisionDescriptorNotFoundException, ObjectLockedException;
    
}
