/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/content/NodeProperty.java,v 1.5 2001/06/20 18:17:27 remm Exp $
 * $Revision: 1.5 $
 * $Date: 2001/06/20 18:17:27 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.slide.content;

import java.io.Serializable;
import java.util.Vector;
import java.util.Enumeration;
import org.apache.slide.security.NodePermission;
import org.apache.slide.util.Messages;
import org.apache.slide.common.ObjectValidationFailedException;

/**
 * Node property class
 * 
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.5 $
 */
public final class NodeProperty implements Serializable, Cloneable {
    
    
    // -------------------------------------------------------------- Constants
    
    
    public static final String DEFAULT_NAMESPACE = "DAV:";
    public static final String SLIDE_NAMESPACE =
        "http://jakarta.apache.org/slide/";
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * Constructor.
     * 
     * @param name Name
     * @param value Value
     */
    public NodeProperty(String name, Object value) {
        setName(name);
        setValue(value);
        this.namespace = DEFAULT_NAMESPACE;
        this.type = new String();
        this.protectedProperty = false;
        this.permissions = new Vector();
    }
    
    
    /**
     * Constructor.
     * 
     * @param name Name
     * @param value Value
     * @param standard Standard
     */
    public NodeProperty(String name, Object value, boolean protectedProperty) {
        this(name, value);
        this.protectedProperty = true;
    }
    
    
    /**
     * Constructor.
     * 
     * @param name Name
     * @param value Value
     * @param namespace Namespace
     */
    public NodeProperty(String name, Object value, String namespace) {
        this(name, value);
        setNamespace(namespace);
    }
    
    
    /**
     * Constructor.
     * 
     * @param name Name
     * @param value Value
     * @param namespace Namespace
     * @param type Type info
     * @param protectedProperty Protected property
     */
    public NodeProperty(String name, Object value, String namespace, 
                        String type, boolean protectedProperty) {
        this(name, value, namespace);
        setType(type);
        this.protectedProperty = protectedProperty;
    }
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Standard.
     */
    private boolean protectedProperty;
    
    
    /**
     * Property name.
     */
    private String name;
    
    
    /**
     * Namespace of the property.
     */
    private String namespace;
    
    
    /**
     * Property value.
     */
    private Object value;
    
    
    /**
     * Value Type. If the value is stored as a String representation of the 
     * value, the type field is used.
     * FIXME : Remove that
     */
    private String type;
    
    
    /**
     * Permission list.
     */
    private Vector permissions;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Protected accessor.
     * 
     * @return boolean true
     */
    public boolean isProtected() {
        return this.protectedProperty;
    }
    
    
    /**
     * Property name accessor.
     * 
     * @return String property name
     */
    public String getName() {
        return this.name;
    }
    
    
    /**
     * Property name mutator.
     * 
     * @param name Name
     */
    void setName(String name) {
        if (name == null) {
            this.name = new String();
        } else {
            this.name = name;
        }
    }
    
    
    /**
     * Namespace accessor.
     * 
     * @return String definition
     */
    public String getNamespace() {
        return this.namespace;
    }
    
    
    /**
     * Namespace mutator.
     * 
     * @param definition Namepsace
     */
    void setNamespace(String namespace) {
        if (namespace == null) {
            this.namespace = DEFAULT_NAMESPACE;
        } else {
            this.namespace = namespace;
        }
    }
    
    
    /**
     * Value accessor.
     * 
     * @return Object value
     */
    public Object getValue() {
        return value;
    }
    
    
    /**
     * Value mutator.
     * 
     * @param value Value
     */
    void setValue(Object value) {
        if (value == null) {
            this.value = new String();
        } else {
            this.value = value;
        }
    }
    
    
    /**
     * Type accessor.
     * 
     * @return String type
     */
    public String getType() {
        return type;
    }
    
    
    /**
     * Type mutator.
     * 
     * @param type Type
     */
    void setType(String type) {
        if (type == null) {
            this.type = new String();
        } else {
            this.type = type;
        }
    }
    
    
    /**
     * Add permission.
     * 
     * @param permission Permission
     */
    public void addPermission(NodePermission permission) {
        permissions.addElement(permission);
    }
    
    
    /**
     * Remove permission.
     * 
     * @param permission Permission to remove
     */
    public void removePermission(NodePermission permission) {
        permissions.removeElement(permission);
    }
    
    
    /**
     * Enumerate permissions.
     * 
     * @return Enumeration permissions
     */
    public Enumeration enumeratePermissions() {
        return permissions.elements();
    }
    
    
    // --------------------------------------------------------- Object Methods
    
    
    /**
     * Clone.
     * 
     * @return Object clone
     */
    NodeProperty cloneObject() {
        NodeProperty result = null;
        try {
            result = (NodeProperty) super.clone();
        } catch(CloneNotSupportedException e) {
        }
        return result;
    }
    
    
    /**
     * Hash Code.
     * 
     * @return int Hash code value
     */
    public int hashCode() {
        return getName().hashCode();
    }
    
    
    /**
     * String representation of the permission.
     * <p/>
     * Format : ObjectUri-SubjectUri-ActionUri-InheritanceFlag
     * 
     * @return String String representation
     */
    public String toString() {
        return getName();
    }
    
    
    /**
     * Equals.
     * 
     * @param obj Object to test
     * @return boolean True if the two object are equal :
     * <li>obj is of type NodeProperty and is not null</li>
     * <li>The property names are equal</li>
     */
    public boolean equals(Object obj) {
        boolean result = false;
        if ((obj != null) && (obj instanceof NodeProperty)) {
            NodeProperty property = (NodeProperty) obj;
            result = (this.name.equals(property.getName()));
        }
        return result;
    }
    
    
    /**
     * Validate.
     */
    public void validate() {
        
        if (name == null)
            throw new ObjectValidationFailedException
                (Messages.message(NodeProperty.class.getName() + ".nullName"));
        
        if (namespace == null)
            throw new ObjectValidationFailedException
                (Messages.message
                 (NodeProperty.class.getName() + ".nullNamespace"));
        
        if (value == null)
            throw new ObjectValidationFailedException
                (Messages.message
                 (NodeProperty.class.getName() + ".nullValue"));
        
    }
    
    
}
