/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/content/NodeRevisionContent.java,v 1.8 2001/05/21 15:42:56 juergen Exp $
 * $Revision: 1.8 $
 * $Date: 2001/05/21 15:42:56 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.content;

import java.io.*;
import java.util.Date;
import java.util.List;
import java.util.ArrayList;
import org.apache.slide.common.*;
import org.apache.slide.util.Messages;

/**
 * Encapsultes the contents of a revision.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @author Juergen Pill
 * @version $Revision: 1.8 $
 */
public final class NodeRevisionContent implements Serializable {
    
    
    // -------------------------------------------------------------- Constants
    
    
    private static final int CHUNK = 1024*4;
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Content.
     */
    private char[] content = null;
    private byte[] contentBytes = null;
    
    
    /**
     * Reader.
     */
    private transient Reader reader = null;
    
    
    /**
     * Input stream.
     */
    private transient InputStream inputStream = null;
    
    
    // ------------------------------------------------------------- Properties
    
    
    /**
     * Content accessor.
     *
     * @return char[] Content
     */
    public char[] getContent() {
        char[] result = null;
        if (content != null) {
            result = content;
            inputStream = null;
            reader = null;
        }
        else if (reader != null) {
            try {
                content = readFromReader(reader);
            } catch (IOException e) {
                e.printStackTrace();
            }
            result = content;
            inputStream = null;
            reader = null;
        }
        else if (contentBytes != null) {
            content = new String(contentBytes).toCharArray();
            result = content;
            inputStream = null;
            reader = null;
        }
        else if (inputStream != null) {
            try {
                contentBytes = readFromStream(inputStream);
            } catch (IOException e) {
                e.printStackTrace();
            }
            content = new String(contentBytes).toCharArray();
            result = content;
            inputStream = null;
            reader = null;
        }
        return result;
    }
    
    /**
     * Content accessor.
     *
     * @return char[] Content
     */
    public byte[] getContentBytes() {
        byte[] result = null;
        if (contentBytes != null) {
            result = contentBytes;
            inputStream = null;
            reader = null;
        }
        else if (inputStream != null) {
            try {
                contentBytes = readFromStream(inputStream);
            } catch (IOException e) {
                e.printStackTrace();
            }
            result = contentBytes;
            inputStream = null;
            reader = null;
        }
        else if (content != null) {
            contentBytes = new String(content).getBytes();
            result = contentBytes;
            inputStream = null;
            reader = null;
        }
        else if (reader != null) {
            try {
                content = readFromReader(reader);
            } catch (IOException e) {
                e.printStackTrace();
            }
            contentBytes = new String(content).getBytes();
            result = contentBytes;
            inputStream = null;
            reader = null;
        }
        return result;
    }
    
    
    
    /**
     * Content accessor.
     *
     * @return Reader
     */
    public Reader readContent()
        throws IOException {
        Reader result = null;
        if (reader != null) {
            result = reader;
            inputStream = null;
        }
        else if (content != null) {
            result = new CharArrayReader(content);
            inputStream = null;
            reader = null;
        }
        else if (inputStream != null) {
            result = new InputStreamReader(inputStream);
            reader = null;
        }
        else if (contentBytes != null) {
            result = new CharArrayReader(new String(contentBytes).toCharArray());
            inputStream = null;
            reader = null;
        }
        return result;
    }
    
    
    /**
     * Content accessor.
     *
     * @return InputStream
     */
    public InputStream streamContent()
        throws IOException {
        InputStream result = null;
        if (inputStream != null) {
            result = inputStream;
            content = null;
            reader = null;
        }
        else if (contentBytes != null) {
            result = new ByteArrayInputStream( contentBytes );
            reader = null;
            inputStream = null;
        }
//      else if( reader != null ) {
//            result = ???;
//            inputStream = null;
//      }
        else if (content != null) {
//            Class StringBufferInputStream is deprecated !!
//            result = new StringBufferInputStream(new String(content));
            result = new ByteArrayInputStream( new String(content).getBytes() );
            reader = null;
            inputStream = null;
        }
        return result;
    }
    
    
    /**
     * Content mutator.
     *
     * @param content New content
     */
    public void setContent(byte[] contentBytes) {
        this.contentBytes = contentBytes;
        this.reader = null;
        this.inputStream = null;
        this.content = null;
    }
    
    /**
     * Content mutator.
     *
     * @param content New content
     */
    public void setContent(char[] content) {
        this.content = content;
        this.reader = null;
        this.inputStream = null;
        this.contentBytes = null;
    }
    
    
    /**
     * Content mutator.
     *
     * @param reader New reader
     */
    public void setContent(Reader reader) {
        this.reader = reader;
        this.inputStream = null;
        this.content = null;
        this.contentBytes = null;
    }
    
    
    /**
     * Content mutator.
     *
     * @param inputStream New input stream
     */
    public void setContent(InputStream inputStream) {
        this.inputStream = inputStream;
        this.reader = null;
        this.content = null;
        this.contentBytes = null;
    }
    
    
    // TODO : Add real serialization support

    
    /**
     * Validate.
     */
    public void validate() {
        
        if ((content == null) && (contentBytes == null) && (reader == null) && (inputStream == null))
            throw new ObjectValidationFailedException
                (Messages.message
                 (NodeRevisionContent.class.getName() + ".noContent"));
        
    }
    
    
    // -------------------------------------------------------- Private Methods
    
    
    /**
     * Read the data from the stream and return the result in a byte array.
     * Return null in case of an error.
     */
    public static  byte[] readFromStream(InputStream inputStream) throws IOException {
        byte[] chunk;
        byte[] all;
        int chunkLen;
        int allLen;
        List chunks;
        int i;
        int max;
        int ofs;
        
        allLen = 0;
        chunk = new byte[CHUNK];
        chunkLen = inputStream.read(chunk);
        chunks = new ArrayList();
        while (chunkLen != -1) {
            chunks.add(new Integer(chunkLen));
            chunks.add(chunk);
            allLen += chunkLen;
            chunk = new byte[CHUNK];
            chunkLen = inputStream.read(chunk);
        }

        all = new byte[allLen];
        ofs = 0;
        max = chunks.size();
        for (i = 0; i < max; i += 2) {
            chunkLen = ((Integer) chunks.get(i)).intValue();
            chunk = (byte[]) chunks.get(i + 1);
            System.arraycopy(chunk, 0, all, ofs, chunkLen);
            ofs += chunkLen;
        }
        return all;
    }
    
    
    
    
    /**
     * Read the data from the reader and return the result in a char array.
     * Return null in case of an error.
     */
    private char[] readFromReader(Reader reader) throws IOException {
        char[] chunk;
        char[] all;
        int chunkLen;
        int allLen;
        List chunks;
        int i;
        int max;
        int ofs;
        
        allLen = 0;
        chunk = new char[CHUNK];
        chunkLen = reader.read(chunk);
        chunks = new ArrayList();
        while (chunkLen != -1) {
            chunks.add(new Integer(chunkLen));
            chunks.add(chunk);
            allLen += chunkLen;
            chunk = new char[CHUNK];
            chunkLen = reader.read(chunk);
        }
        
        all = new char[allLen];
        ofs = 0;
        max = chunks.size();
        for (i = 0; i < max; i += 2) {
            chunkLen = ((Integer) chunks.get(i)).intValue();
            chunk = (char[]) chunks.get(i + 1);
            System.arraycopy(chunk, 0, all, ofs, chunkLen);
            ofs += chunkLen;
        }
        return all;
    }
    
    
}
