/*
 * $Header: /home/cvspublic/jakarta-slide/src/share/org/apache/slide/security/Security.java,v 1.16 2001/05/16 12:06:12 juergen Exp $
 * $Revision: 1.16 $
 * $Date: 2001/05/16 12:06:12 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.security;

import java.util.Enumeration;
import org.apache.slide.common.*;
import org.apache.slide.structure.*;
import org.apache.slide.authenticate.CredentialsToken;
import org.apache.slide.util.Configuration;

/**
 * Security helper.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 * @version $Revision: 1.16 $
 */
public interface Security {
    
    
    // ------------------------------------------------------ Interface Methods
    
    
    /**
     * Enumerates permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    Enumeration enumeratePermissions(SlideToken token, ObjectNode object)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Enumerates permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    Enumeration enumeratePermissions(SlideToken token, String object)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Set a new set of permissions on an object.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void setPermissions(SlideToken token, String object, 
                        Enumeration permissions)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param activity Activity which the actor can perform
     * @param collection Collection on which the actor can perform
     * the specified action
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void grantPermission(SlideToken token, ObjectNode object,
                         SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param permission New permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void grantPermission(SlideToken token, NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Grants a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @param inheritable Create an inheritable permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void grantPermission(SlideToken token, ObjectNode object,
                         SubjectNode subject, ActionNode action,
                         boolean inheritable)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param activity Activity which the actor can perform
     * @param collection Collection on which the actor can perform
     * the specified action
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void denyPermission(SlideToken token, ObjectNode object,
                        SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param permission New permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void denyPermission(SlideToken token, NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Deny a new permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is granted
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @param inheritable Create an inheritable permission
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void denyPermission(SlideToken token, ObjectNode object,
                        SubjectNode subject, ActionNode action,
                        boolean inheritable)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Revokes a permission.
     *
     * @param token Credentials token
     * @param object Object on which permission is revoked
     * @param subject Subject who can perform the action
     * @param action Action which can be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void revokePermission(SlideToken token, ObjectNode object,
                          SubjectNode subject, ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
    
    
    /**
     * Revokes a permission.
     *
     * @param token Credentials token
     * @param permission Permission to be removed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    public void revokePermission(SlideToken token, NodePermission permission)
        throws ServiceAccessException, ObjectNotFoundException,
        AccessDeniedException;
            
    
     
    
    /**
     * Check if the credentials given grants permission to perform
     * the specified action on the specified subject.
     *
     * @param token Credentials token
     * @param object Object on which the action is performed
     * @param action Action performed
     * @exception ServiceAccessException DataSource access error
     * @exception AccessDeniedException The credentials does not grant
     * the permission to perform the specified action
     */
    void checkCredentials(SlideToken token, ObjectNode object,
                          ActionNode action)
        throws ServiceAccessException, AccessDeniedException;
    
    
    /**
     * Check whether or not an actor can perform the specified activity
     * on a collection.
     *
     * @param object Object on which access is tested
     * @param subject Subject who seeks to perform the action
     * @param action Action which is to be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     * @exception AccessDeniedException Insufficent credentials
     */
    void checkPermission(ObjectNode object, SubjectNode subject,
                         ActionNode action)
        throws ServiceAccessException, AccessDeniedException,
        ObjectNotFoundException;
    
    
    /**
     * Check whether or not an actor can perform the specified activity
     * on a collection.
     *
     * @param object Object on which access is tested
     * @param subject Subject who seeks to perform the action
     * @param action Action which is to be performed
     * @return true if the action can be performed
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    boolean hasPermission(ObjectNode object, SubjectNode subject,
                          ActionNode action)
        throws ServiceAccessException, ObjectNotFoundException;
    
    
    /**
     * Check whether or not the current user has the specified role.
     *
     * @param token Credentials token
     * @param role Role
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    boolean hasRole(SlideToken token, String role)
        throws ServiceAccessException, ObjectNotFoundException;
    
    
    /**
     * Check whether or not the current user has the specified role.
     *
     * @param object Object node
     * @param role Role
     * @exception ServiceAccessException DataSource access error
     * @exception ObjectNotFoundException Specified object was not found
     * in the DataSource
     */
    boolean hasRole(ObjectNode object, String role)
        throws ServiceAccessException, ObjectNotFoundException;
    
    
    /**
     * Return the list of roles the specified node has.
     *
     * @param object Object node
     */
    Enumeration getRoles(ObjectNode object);
    
    
    /**
     * Return the list of roles the specified token has.
     *
     * @param token Credentials token
     */
    Enumeration getRoles(SlideToken token)
        throws ServiceAccessException, ObjectNotFoundException;
    
    
    /**
     * Get the principal associated with the credentials token.
     *
     * @param token Creadentials token
     */
    ObjectNode getPrincipal(SlideToken token)
        throws ServiceAccessException, ObjectNotFoundException;
    
    
}
