/*
 * $Header: /home/cvspublic/jakarta-slide/src/tests/transaction/PutTestCase.java,v 1.2 2001/05/01 21:30:14 remm Exp $
 * $Revision: 1.2 $
 * $Date: 2001/05/01 21:30:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package transaction;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import junit.ui.TestRunner;

import org.apache.commons.httpclient.Cookie;
import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpClient;

import org.apache.commons.httpclient.methods.PutMethod;

// Slide
import org.apache.webdav.lib.*;
import org.apache.webdav.lib.methods.*;

// Java
import java.io.*;
import java.util.*;

/**
 * Test cases for the PUT method
 *
 * @author Juergen Pill
 * @version $Revision: 1.2 $
 */
public class PutTestCase extends TestCase {


    private static final String destination = "/slide/Patients/myFile.txt";
    private static final String fileContent = "<patient1>\n <name>\n  <surname>pat rec 1</surname>\n  <firstname>1</firstname>\n  </name>\n</patient1>\n";
    private static final int    portNumber  = 8080;

	
    /**
     * Class initialisation
     */
	public PutTestCase(String name) {
        super(name);
    }


	/**********************************************************************/
	/*                                                                    */
	/* public methods which are called by Junit                           */
	/*                                                                    */
	/**********************************************************************/
	
	

    /**
     * Test a simple PUT command
     */
    public void test1PutCommand() throws Exception {
    	System.out.println("Starting test1PutCommand");
		excutePut(1);
		excutePut(1);
		excutePut(1);
		excutePut(1);
		excutePut(1);
		excutePut(1);
    }

    /**
     * Test 10 simple PUT command
     */
    public void test10PutCommand() throws Exception {
    	System.out.println("Starting test10PutCommand");
		excutePut(10);
    }

    /**
     * Test 100 simple PUT command
     */
    public void test100PutCommand() throws Exception {
    	System.out.println("Starting test100PutCommand");
		//excutePut(100);
    }


	

    /**
     * Test a simple PUT command in one thread
     */
    public void testParallel1PutCommand() throws Exception {
    	System.out.println("Starting testParallel1PutCommand");
		parallelExcutePut(1, 1);
		parallelExcutePut(1, 1);
		parallelExcutePut(1, 1);
		parallelExcutePut(1, 1);
		parallelExcutePut(1, 1);
    }

    /**
     * Test 10 simple PUT command in 10 thread
     */
    public void testParallel10PutCommand() throws Exception {
    	System.out.println("Starting testParallel10PutCommand");
		parallelExcutePut(10, 10);
    }

    /**
     * Test 100 simple PUT command in 100 thread
     */
    public void testParallel100PutCommand() throws Exception {
    	System.out.println("Starting testParallel100PutCommand");
		parallelExcutePut(100, 100);
    }


	
	
	
	/**********************************************************************/
	/*                                                                    */
	/* private method which perform the test cases                        */
	/*                                                                    */
	/**********************************************************************/
	
	
	
	

	
	

    /**
     * Execute a multiple test cases in many threads
     */
    private void parallelExcutePut(int threadCount, int putCount) throws Exception {
    	Vector allThreads = new Vector(threadCount);
		for (int i = 1 ; i<=threadCount; i++) {
			allThreads.add(new Thread(new PutRunner(putCount), "parallelExcutePut-"+i));
		}
		for (int i = 0 ; i<threadCount; i++) {
			((Thread)allThreads.elementAt(i)).start();
		}
		for (int i = 0 ; i<threadCount; i++) {
			((Thread)allThreads.elementAt(i)).join();
		}
	}
	

    /**
     * Execute a individual test case in many threads
     */
    private void parallelExcutePut(int threadCount) throws Exception {
		for (int i = 1 ; i<=threadCount; i++) parallelExcutePut(threadCount, 1);
	}
	
	
	

    /**
     * Execute multiple test cases
     */
    private void excutePut(int putCount) throws Exception {
		for (int i = 1 ; i<=putCount; i++) excutePut(destination+i);
	}
	
	
	
    /**
     * Execute a individual test case
     */
    private void excutePut(String resourceName) throws Exception {
		long startTime = System.currentTimeMillis();
	    HttpClient client = new HttpClient();
		PutMethod pm = new PutMethod();
		try {
//        	client.setDebug(1);
			client.startSession("localhost", portNumber);
			client.setCredentials(new Credentials("guest", "guest"));
			pm.setPath( resourceName );
			pm.sendData( new StringBufferInputStream(fileContent));
			client.executeMethod( pm );
			
			System.out.println( "PUT  to " + resourceName + " : " + pm.getStatusCode() +
								"  " + pm.getStatusText() +
								" (time: " + (System.currentTimeMillis() - startTime) + " ms)" );
		}
		catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
		
		assertEquals("Response code ", 200, pm.getStatusCode());
	}
	
	
	
	/**************************************************************/
	/* Inner class to help the thread to execute the put function */
	/**************************************************************/
	public class PutRunner implements Runnable{
		int putCount;
		public PutRunner(int putCount) {
			this.putCount = putCount;
		}
		public void run() {
			try {excutePut(putCount);}
			catch (Exception e) {e.printStackTrace();};}
	}
	
	
	
}
	
	

