/*
 * $Header: /home/cvspublic/jakarta-slide/src/webdav/server/org/apache/slide/webdav/method/MkcolMethod.java,v 1.9 2001/04/25 01:57:14 remm Exp $
 * $Revision: 1.9 $
 * $Date: 2001/04/25 01:57:14 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.webdav.method;

import java.security.Principal;
import java.io.*;
import java.util.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.util.WebdavStatus;
import org.apache.slide.common.*;
import org.apache.slide.webdav.*;
import org.apache.slide.structure.*;
import org.apache.slide.content.*;
import org.apache.slide.lock.*;
import org.apache.slide.security.AccessDeniedException;

/**
 * MKCOL method.
 *
 * @author <a href="mailto:remm@apache.org">Remy Maucherat</a>
 */
public class MkcolMethod extends WebdavMethod {
    
    
    // -------------------------------------------------------------- Constants
    
    
    // ----------------------------------------------------- Instance Variables
    
    
    /**
     * Collection name.
     */
    private String colName;
    
    
    // ----------------------------------------------------------- Constructors
    
    
    /**
     * GET Method constructor.
     *
     * @param token Namespace access token
     * @param req HTTP request
     * @param resp HTTP response
     */
    public MkcolMethod(GenericServlet servlet, NamespaceAccessToken token,
                       HttpServletRequest req, HttpServletResponse resp) {
    super(servlet, token, req, resp);
    }
    
    
    // ------------------------------------------------------ Protected Methods
    
    
    /**
     * Parse XML request.
     */
    protected void parseRequest()
        throws WebdavException {
        colName = requestUri;
        if (colName == null) {
            colName = "/";
        }
    }
    
    
    /**
     * Execute request.
     *
     * @exception WebdavException Bad request
     */
    protected void executeRequest()
        throws WebdavException {
        
        SubjectNode collection = new SubjectNode();
        NodeRevisionDescriptor revisionDescriptor =
            new NodeRevisionDescriptor(0);
        
        NodeProperty property = null;
        
        // Resource type
        property = new NodeProperty("resourcetype", "<collection/>", true);
        revisionDescriptor.setProperty(property);
        
        // Creation date
        revisionDescriptor.setCreationDate(new Date());
        
        // Last modification date
        revisionDescriptor.setLastModified(new Date());
        
        // Display name
        property = new NodeProperty("getcontentlength", "0", true);
        revisionDescriptor.setProperty(property);
        
        // Source
        property = new NodeProperty("source", "", true);
        revisionDescriptor.setProperty(property);
        
        if (isMsProprietarySupport()) {
            
            // Name
            /*
              property = new NodeProperty("name", colName, "ms", "MICROSOFT");
              properties.setProperty(property);
            */
            
            // ParentName
            // TODO : Fix this
            /*
              property = new NodeProperty("parentname", colName, "ms",
              "MICROSOFT");
              properties.setProperty(property);
            */
            
            // Href
            /*
              property = new NodeProperty("href", colName, "ms", "MICROSOFT");
              properties.setProperty(property);
            */
            
            // Is hidden
            property = new NodeProperty("ishidden", "0", "MICROSOFT");
            revisionDescriptor.setProperty(property);
            
            // Is collection
            property = new NodeProperty("iscollection", "1", "MICROSOFT");
            revisionDescriptor.setProperty(property);
            
            // Is read only
            property = new NodeProperty("isreadonly", "0", "MICROSOFT");
            revisionDescriptor.setProperty(property);
            
            // Last accessed
            property = new NodeProperty("lastaccessed",
                                        (new Date()).toString(), "MICROSOFT");
            revisionDescriptor.setProperty(property);
            
        }
        
        // If everything is ok : 201 - Created / OK
        resp.setStatus(WebdavStatus.SC_CREATED);
    
        try {
            structure.create(slideToken, collection, colName);
            content.create(slideToken, colName, revisionDescriptor, null);
        } catch (ObjectNotFoundException e) {
        resp.setStatus(WebdavStatus.SC_CONFLICT);
        throw new WebdavException(WebdavStatus.SC_CONFLICT);
    } catch (ObjectLockedException e) {
        resp.setStatus(WebdavStatus.SC_LOCKED);
        throw new WebdavException(WebdavStatus.SC_LOCKED);
    } catch (RevisionAlreadyExistException e) {
        resp.setStatus(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
        throw new WebdavException(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
    } catch (ServiceAccessException e) {
        resp.setStatus(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
        throw new WebdavException(WebdavStatus.SC_INTERNAL_SERVER_ERROR);
    } catch (LinkedObjectNotFoundException e) {
        resp.setStatus(WebdavStatus.SC_NOT_FOUND);
        throw new WebdavException(WebdavStatus.SC_NOT_FOUND);
    } catch (ObjectAlreadyExistsException e) {
            resp.setStatus(WebdavStatus.SC_METHOD_NOT_ALLOWED);
            throw new WebdavException(WebdavStatus.SC_METHOD_NOT_ALLOWED, false);
        } catch (AccessDeniedException e) {
        resp.setStatus(WebdavStatus.SC_FORBIDDEN);
        throw new WebdavException(WebdavStatus.SC_FORBIDDEN);
    }
    
    // 415 - Unsupported Media Type
    // TODO : Has something to do with the body of the request ...
    // WebDAV RFC is vague on the subject.
    
    // 507 - Insufficient storage
    // Would be returned as a ServiceAccessException, so it would
        // return an Internal Server Error, which is probably acceptable.
    
    // TODO : Initialize and create collection's properties.
    
    }
    
    
    /**
     * Returns true
     */
    protected boolean methodNeedsTransactionSupport() {
        return true;
    }
    
    
}

